#include "Functions.h"
#include <cstdlib>

void clearScreen() {
#ifdef _WIN32
    system("cls");
#else
    system("clear");
#endif
};

void pauseScreen() {
#ifdef _WIN32
    system("pause");
#else
    system("read -p 'Press Enter to continue...' var");
#endif
};

class Menu {
public:
    Menu(const std::string& filename) : m_filename(filename) {
        loadMenuItems();
    }

    void displayMainMenu() {
        for (const auto& item : menuItems) {
            std::cout << item << std::endl;
        }
    }

private:
    std::vector<std::string> menuItems;
    std::string m_filename;

    void loadMenuItems() {
        std::ifstream file(m_filename);
        if (!file.is_open()) {
            std::cerr << "Unable to open file: " << m_filename << std::endl;
            return;
        }

        std::string line;
        while (std::getline(file, line)) {
            menuItems.push_back(line);
        }

        file.close();
    }
};


template <class Container>
class Application {
public:
    void run() {
        clearScreen();
        Menu menu("menu.txt");
        menu.displayMainMenu();
        int choice;
        std::cout << "Enter your choice: ";
        std::cin >> choice;
        switch (choice)
        {
        case 1:
            clearScreen(); createDataMenu(); break;
        case 2:
            clearScreen(); loadDataMenu(); break;
        case 3:
            clearScreen(); modifyDataMenu(); break;
        case 4:
            clearScreen(); outputResultMenu(); break;
        case 5:
            std::cout << "Exiting the application" << std::endl; pauseScreen(); return;
        default:
            clearScreen(); std::cout << "Invalid choice." << std::endl; pauseScreen();
        }
    };
private:
    Container m_cont;
    std::string m_filename;
    void createDataMenu()
    {
        int choice;
        std::cout << "Select data creation method:" << std::endl;
        std::cout << "1. Using File Streams" << std::endl;
        std::cout << "2. Using Generate and Copy" << std::endl;
        std::cin >> choice;
        std::cout << "Enter the filename: ";
        std::cin >> m_filename;
        switch (choice)
        {
        case 1:
            clearScreen(); createDataStream(m_filename); pauseScreen(); break;
        case 2:
            clearScreen(); createDataGenerate(m_filename); pauseScreen(); break;
        default:
            clearScreen(); std::cout << "Invalid choice." << std::endl; pauseScreen();
        }
    }

    void loadDataMenu()
    {
        int choice;
        std::cout << "Select data loading method:" << std::endl;
        std::cout << "1. Load Data from File" << std::endl;
        std::cout << "2. Load Data using Iterators" << std::endl;
        std::cin >> choice;
        std::cout << "Enter the filename: ";
        std::cin >> m_filename;
        switch (choice)
        {
        case 1:
        {
            clearScreen(); loadDataStream(m_filename, m_cont); printContainer(m_cont); pauseScreen(); break;
        }
        case 2:
        {
            clearScreen(); loadDataIterator(m_filename, m_cont); printContainer(m_cont); pauseScreen(); break;
        }
        default:
            clearScreen(); std::cout << "Invalid choice." << std::endl; pauseScreen();
        }
    }

    void modifyDataMenu()
    {
        int choice;
        std::cout << "Select data modification method:" << std::endl;
        std::cout << "1. Modify using template function" << std::endl;
        std::cout << "2. Modify using iterators" << std::endl;
        std::cout << "3. Modify using for_each" << std::endl;
        std::cout << "4. Modify using transform" << std::endl;
        std::cin >> choice;
        switch (choice)
        {
        case 1:
        {
            clearScreen(); modifyTemplate(m_cont); printContainer(m_cont); pauseScreen(); break;
        }
        case 2:
        {
            clearScreen(); modifyIter(m_cont.begin(), m_cont.end()); printContainer(m_cont); pauseScreen(); break;
        }
        case 3:
        {
            clearScreen(); modifyForEach(m_cont); printContainer(m_cont); pauseScreen(); break;
        }
        case 4:
        {
            std::cout << "Enter original file" << std::endl;
            std::string origfile;
            std::cin >> origfile;
            std::cout << "Enter final file" << std::endl;
            std::string finfile;
            std::cin >> finfile;
            clearScreen();
            modifyTransform(origfile, finfile);
            pauseScreen();
            break;
        }
        default:
            clearScreen(); std::cout << "Invalid choice." << std::endl; pauseScreen();
        }
    }

    void outputResultMenu()
    {
        int choice;
        std::cout << "Select data output method:" << std::endl;
        std::cout << "1. Output Result to File" << std::endl;
        std::cout << "2. Output Result using Copy" << std::endl;
        std::cin >> choice;
        std::cout << "Enter the output filename: ";
        std::cin >> m_filename;
        switch (choice)
        {
        case 1:
        {
            clearScreen(); outputResultStream(m_filename, m_cont); pauseScreen(); break;
        }
        case 2:
        {
            clearScreen(); outputResultCopy(m_filename, m_cont); pauseScreen(); break;
        }
        default:
            clearScreen(); std::cout << "Invalid choice." << std::endl; pauseScreen();
        }
    }
};
