#include "iostream"
#include "fstream"
#include "list"
#include "deque"
#include "numeric"
#include "vector"
#include "list"
#include "algorithm"
#include "string"

void createDataStream(const std::string& filename) {
    std::srand(std::time(nullptr));
    std::ofstream file(filename, std::ios::trunc);
    if (file.is_open()) {
        for (int i = 0; i < 100; i++) {
            file << rand() % 101 - 50 << " ";
        };
        std::cout << "file was succesfully created using file streams: " << filename << std::endl;
    }
    else
        std::cerr << "incapable to create file" << std::endl;
};

void createDataGenerate(const std::string& filename) {
    std::srand(std::time(nullptr));
    std::vector<int> data(100);
    std::generate(data.begin(), data.end(), []() { return std::rand() % 101 - 50; });
    std::ofstream file(filename, std::ios::trunc);
    if (file.is_open())
    {
        std::copy(data.begin(), data.end(), std::ostream_iterator<int>(file, " "));
        std::cout << "file was succesfully created using generate and copy: " << filename << std::endl;
    }
    else
        std::cerr << "incapable to create file" << std::endl;
};

template <typename Container>
void loadDataStream(const std::string& filename, Container& cont) {
    std::ifstream file(filename);
    using value_type = typename Container::value_type;
    value_type value;
    if (file.is_open()) {
        while (!file.eof())
        {
            while (file >> value)
            {
                cont.push_back(std::move(value));
            }
        };
    }
    else
        std::cerr << "incapable to create file" << std::endl;
};

template <typename Container>
void loadDataIterator(const std::string& filename, Container& cont)
{
    std::ifstream file(filename);
    if (file.is_open())
    {
        auto beg = std::istream_iterator<typename Container::value_type>(file);
        auto end = std::istream_iterator<typename Container::value_type>();
        std::copy(beg, end, std::back_inserter(cont));
    }
    else
        std::cerr << "incapable to open file" << std::endl;
};

template <typename Container>
void modifyTemplate(Container& cont) {
    using value_type = typename Container::value_type;
    std::vector<value_type> negatives;
    std::copy_if(cont.begin(), cont.end(), std::back_inserter(negatives), [](const value_type& elem) {
        return elem < 0;
        });
    value_type halfSum = std::accumulate(negatives.begin(), negatives.end(), value_type{}) / 2;
    std::for_each(cont.begin(), cont.end(), [halfSum](value_type& elem) {
        elem += halfSum;
        });
    value_type sum = std::accumulate(cont.begin(), cont.end(), value_type{});
    value_type average = std::accumulate(cont.begin(), cont.end(), value_type{}, [](int accumulator, int element) {
        return accumulator + std::abs(element); }) / cont.size();
        cont.push_back(std::move(sum));
        cont.push_back(std::move(average));
};

template <typename Iterator>
void modifyIter(Iterator begin, Iterator end) {
    using value_type = typename std::iterator_traits<Iterator>::value_type;
    std::vector<value_type> negatives;
    std::copy_if(begin, end, std::back_inserter(negatives), [](const value_type& elem) {
        return elem < 0;
        });
    value_type halfSum = std::accumulate(negatives.begin(), negatives.end(), value_type{}) / 2;
    std::for_each(begin, end, [halfSum](value_type& elem) {
        elem += halfSum;
        });
};

template <typename Container>
void modifyForEach(Container& cont) {
    using value_type = typename Container::value_type;
    std::vector<value_type> negatives;
    std::copy_if(cont.begin(), cont.end(), std::back_inserter(negatives), [](const value_type& elem) {
        return elem < 0;
        });
    value_type halfSum = std::accumulate(negatives.begin(), negatives.end(), value_type{}) / 2;
    std::for_each(cont.begin(), cont.end(), [halfSum](value_type& elem) {
        elem += halfSum;
        });
};

void modifyTransform(const std::string& in_fname, const std::string& out_fname) {
    std::ifstream fin(in_fname);
    std::ofstream fout(out_fname);
    std::vector<int> vec;
    std::copy(std::istream_iterator<int>(fin), std::istream_iterator<int>(), std::back_inserter(vec));
    std::vector<int> negatives;
    std::copy_if(vec.begin(), vec.end(), std::back_inserter(negatives), [](const int& elem) {
        return elem < 0;
        });
    int halfSum = std::accumulate(negatives.begin(), negatives.end(), 0) / 2;
    std::transform(vec.begin(), vec.end(), std::ostream_iterator<int>(fout, " "), [halfSum](int& elem) {
        return elem + halfSum;
        });
    fout << std::endl;
};

template <typename Container>
void printContainer(const Container& cont)
{
    std::cout << "container contents: ";
    for (auto it = cont.begin(); it != cont.end(); ++it)
        std::cout << *it << " ";
};

template <typename Container>
void outputResultStream(const std::string& filename, const Container& cont)
{
    using value_type = typename Container::value_type;
    std::ofstream outFile(filename);
    if (outFile.is_open())
    {
        for (const value_type& elem : cont)
            outFile << elem << " ";
        outFile << std::endl;
        std::cout << "data output to file: " << filename << std::endl;
    }
    else
        std::cerr << "nncapable to create file" << std::endl;
};

template <typename Container>
void outputResultCopy(const std::string& filename, const Container& cont)
{
    std::ofstream outFile(filename);
    if (outFile.is_open())
    {
        std::copy(cont.begin(), cont.end(), std::ostream_iterator<typename Container::value_type>(outFile, " "));
        std::cout << "data output to file: " << filename << std::endl;
    }
    else
        std::cerr << "incapable to create file" << std::endl;
};