#include "Automobile.h"
#include <cstdlib>
void clearScreen() {
#ifdef _WIN32
    system("cls");
#else
    system("clear");
#endif
};

void pauseScreen() {
#ifdef _WIN32
    system("pause");
#else
    system("read -p 'Press Enter to continue...' var");
#endif
};

class Menu {
public:
    Menu(const std::string& filename) : m_filename(filename) {
        loadMenuItems();
    }

    void displayMainMenu() {
        for (const auto& item : menuItems) {
            std::cout << item << std::endl;
        }
    }

private:
    std::vector<std::string> menuItems;
    std::string m_filename;

    void loadMenuItems() {
        std::ifstream file(m_filename);
        if (!file.is_open()) {
            std::cerr << "Unable to open file: " << m_filename << std::endl;
            return;
        }

        std::string line;
        while (std::getline(file, line)) {
            menuItems.push_back(line);
        }

        file.close();
    }
};

class Application
{
private:
    std::string m_filename;
    std::map<size_t, Automobile> m_auto;
public:
    void run()
    {
        static int id_counter = 0;
        while (true)
        {
            clearScreen();
            Menu menu("menu.txt");
            menu.displayMainMenu();
            uint32_t choice;
            std::cout << "ENTER YOUT CHOICE: ";
            std::cin >> choice;
            switch (choice)
            {
            case 1:
            {
                clearScreen();
                std::string filename;
                std::cout << "ENTER BINARY FILENAME: ";
                std::cin >> filename;
                if (filename != m_filename)
                    m_filename = filename;
                Automobile newAuto;
                newAuto.m_id = id_counter;
                std::cout << "ENTER AUTOMOBILE'S INFO:" << std::endl;
                std::cin >> newAuto;
                addAutomobile(m_auto, newAuto, m_filename);
                id_counter++;
                pauseScreen();
                break;
            }
            case 2:
            {
                clearScreen();
                std::string filename;
                uint32_t id;
                std::cout << "ENTER BINARY FILENAME: ";
                std::cin >> filename;
                if (filename != m_filename)
                    m_filename = filename;
                std::cout << "ENTER AUTOMOBILE'S ID YOU WANT TO REMOVE: ";
                std::cin >> id;
                auto found_auto = std::find_if(
                    m_auto.begin(),
                    m_auto.end(),
                    [id](const auto& car) {
                        return car.first == id;
                    });
                if (found_auto == m_auto.end())
                    std::cout << "THERE'S NO AUTOMOBILES WITH SUCH ID!" << std::endl;
                else
                    removeAutomobile(m_auto, id, m_filename);
                pauseScreen();
                break;
            }
            case 3:
            {
                clearScreen();
                std::string filename;
                Automobile newAuto;
                uint32_t id;
                std::cout << "ENTER BINARY FILENAME: ";
                std::cin >> filename;
                if (filename != m_filename)
                    m_filename = filename;
                std::cout << "ENTER AUTOMOBILE'S ID YOU WANT TO REPLACE: ";
                std::cin >> id;
                auto found_auto = std::find_if(
                    m_auto.begin(),
                    m_auto.end(),
                    [id](const auto& person) {
                        return person.first == id;
                    });
                if (found_auto == m_auto.end())
                    std::cout << "THERE'S NO AUTOMOBILES WITH SUCH ID!" << std::endl;
                else
                {
                    std::cout << "ENTER AUTOMOBILE'S INFO YOU WANT TO REPLACE BY: " << std::endl;
                    newAuto.m_id = id;
                    std::cin >> newAuto;
                    replaceAutomobile(m_auto, id, newAuto, m_filename);
                }
                pauseScreen();
                break;
            }
            case 4:
            {
                clearScreen();
                std::string filename;
                std::string outputFilename;
                std::string model;
                std::cout << "ENTER BINARY FILENAME: ";
                std::cin >> filename;
                if (filename != m_filename)
                    m_filename = filename;
                std::cout << "ENTER MODEL OF AUTOMOBILE TO SEARCH BY: " << std::endl;
                std::cin >> model;
                std::cout << "ENTER OUTPUT FILENAME: ";
                std::cin >> outputFilename;
                searchByAutomobileModel(m_auto, outputFilename, model);
                pauseScreen();
                break;
            }
            case 5:
            {
                clearScreen();
                std::string filename;
                std::string outputFilename;
                std::string fuel;
                std::cout << "ENTER BINARY FILENAME: ";
                std::cin >> filename;
                if (filename != m_filename)
                    m_filename = filename;
                std::cout << "ENTER FUEL GRADE TO SEARCH BY: " << std::endl;
                std::cin >> fuel;
                std::cout << "ENTER OUTPUT FILENAME: ";
                std::cin >> outputFilename;
                searchByFuelGrade(m_auto, outputFilename, fuel);
                pauseScreen();
                break;
            }
            case 6:
            {
                clearScreen();
                std::string filename;
                std::string outputFilename;
                size_t engine;
                std::cout << "ENTER BINARY FILENAME: ";
                std::cin >> filename;
                if (filename != m_filename)
                    m_filename = filename;
                std::cout << "ENTER ENGINE POWER TO SEARCH BY: " << std::endl;
                std::cin >> engine;
                std::cout << "ENTER OUTPUT FILENAME: ";
                std::cin >> outputFilename;
                searchByEnginePower(m_auto, outputFilename, engine);
                pauseScreen();
                break;
            }
            case 7:
            {
                clearScreen();
                std::string filename;
                std::string outputTextFilename;
                std::cout << "ENTER BINARY FILENAME: ";
                std::cin >> filename;
                if (filename != m_filename)
                    m_filename = filename;
                fileToContainer(m_filename, m_auto);
                std::cout << "ENTER OUTPUT FILENAME: ";
                std::cin >> outputTextFilename;
                convertBinaryToTextFile(m_auto, outputTextFilename);
                pauseScreen();
                break;
            }
            case 8:
                std::cout << "EXCITING THE APPLICATION " << std::endl;
                pauseScreen();
                return;
            default:
                clearScreen(); std::cout << "Invalid choice." << std::endl; pauseScreen();
            }
        }
    }
};
